from celery import shared_task
from celery.utils.log import get_task_logger

from . import download

logger = get_task_logger(__name__)


@shared_task(ignore_result=True)
def update_geoipdb(skip_city=False, skip_country=False, skip_md5=False):
    download(
        skip_city=skip_city, skip_country=skip_country, skip_md5=skip_md5, logger=logger
    )



import gzip
import hashlib
import random
import string
import sys
from io import BytesIO

import geoipdb_loader
import pytest
from django.core import management
from django.core.exceptions import ImproperlyConfigured
from django.utils import six

PY2 = sys.version_info[0] == 2


if PY2:
    import mock
else:
    from unittest import mock


@pytest.fixture
def random_bytes():
    return six.b(
        "".join(
            random.choice(string.ascii_uppercase + string.digits + "\n")
            for _ in range(100)
        )
    )


def test_match_md5(monkeypatch, random_bytes):
    md5 = hashlib.md5()
    md5.update(random_bytes)
    md5sum = md5.hexdigest()
    urlopen_mock = mock.Mock(return_value=mock.Mock(read=lambda: six.b(md5sum)))
    monkeypatch.setattr("django.utils.six.moves.urllib.request.urlopen", urlopen_mock)
    fp = BytesIO(random_bytes)
    assert geoipdb_loader._match_md5(fp, "someurl")
    urlopen_mock.assert_called_once_with("someurl")
    assert fp.read() == random_bytes


@pytest.mark.parametrize("is_md5_match", [False, True])
@pytest.mark.parametrize("skip_md5", [False, True])
def test_download_file(
    monkeypatch, tmpdir, settings, random_bytes, is_md5_match, skip_md5
):
    def create_gzip(url, filename):
        with gzip.open(filename, "wb") as f:
            f.write(random_bytes)

    settings.GEOIP_PATH = str(tmpdir)
    monkeypatch.setattr(
        "django.utils.six.moves.urllib.request.urlretrieve",
        mock.Mock(side_effect=create_gzip),
    )
    match_md5_mock = mock.Mock(return_value=is_md5_match)
    monkeypatch.setattr("geoipdb_loader._match_md5", match_md5_mock)
    if is_md5_match or skip_md5:
        geoipdb_loader._download_file("somedb.mmdb.gz", skip_md5=skip_md5)
        assert open(str(tmpdir.join("somedb.mmdb")), "rb").read() == random_bytes
    else:
        with pytest.raises(ValueError) as e:
            geoipdb_loader._download_file("somedb.mmdb.gz", skip_md5=skip_md5)
        assert str(e.value) == "md5 of %s doesn't match the signature." % tmpdir.join(
            "somedb.mmdb.gz"
        )
    if skip_md5:
        assert not match_md5_mock.call_count
    else:
        assert match_md5_mock.call_count == 1


@pytest.mark.parametrize("skip_city", [False, True])
@pytest.mark.parametrize("skip_country", [False, True])
@pytest.mark.parametrize("skip_md5", [False, True])
def test_command(monkeypatch, settings, tmpdir, skip_city, skip_country, skip_md5):
    settings.GEOIP_PATH = str(tmpdir)
    download_file_mock = mock.Mock()
    monkeypatch.setattr("geoipdb_loader._download_file", download_file_mock)
    out = six.StringIO()
    args = []
    if skip_city:
        args.append("--skip-city")
    if skip_country:
        args.append("--skip-country")
    if skip_md5:
        args.append("--skip-md5")
    management.call_command("download_geoipdb", stdout=out, *args)
    if not skip_city:
        download_file_mock.assert_any_call(
            maxmind_filename="GeoLite2-City.mmdb.gz",
            skip_md5=skip_md5,
            local_filename=None,
        )
    if not skip_country:
        download_file_mock.assert_any_call(
            maxmind_filename="GeoLite2-Country.mmdb.gz",
            skip_md5=skip_md5,
            local_filename=None,
        )
    assert download_file_mock.call_count == int(not skip_city) + int(not skip_country)


def test_download_edge_cases(monkeypatch, settings, tmpdir):
    with pytest.raises(ImproperlyConfigured) as e:
        geoipdb_loader.download(skip_city=True, skip_country=True)
    assert str(e.value) == "GEOIP_PATH must be configured in settings."
    log_mock = mock.Mock()
    settings.GEOIP_PATH = str(tmpdir)
    monkeypatch.setattr("geoipdb_loader._get_logger", lambda: log_mock)
    geoipdb_loader.download(skip_city=True, skip_country=True)
    log_mock.warn.assert_called_once_with("Nothing to download.")


@pytest.mark.parametrize("paths", [(None, None), ("country.dat", "city.mmdb")])
def test_geoipdb_version(monkeypatch, settings, tmpdir, random_bytes, paths):
    def create_gzip(url, filename):
        suffix = "city" if "city" in filename.lower() else "country"
        with gzip.open(filename, "wb") as f:
            f.write(random_bytes + six.b(suffix))

    settings.GEOIP_COUNTRY, settings.GEOIP_CITY = paths
    settings.GEOIP_PATH = str(tmpdir)
    monkeypatch.setattr(
        "django.utils.six.moves.urllib.request.urlretrieve",
        mock.Mock(side_effect=create_gzip),
    )
    geoipdb_loader.download(skip_md5=True)

    assert open(
        str(tmpdir.join(settings.GEOIP_COUNTRY or "GeoLite2-Country.mmdb")), "rb"
    ).read() == random_bytes + six.b("country")
    assert open(
        str(tmpdir.join(settings.GEOIP_CITY or "GeoLite2-City.mmdb")), "rb"
    ).read() == random_bytes + six.b("city")


def test_download_on_version(monkeypatch, settings, tmpdir):
    settings.GEOIP_PATH = str(tmpdir)
    download_file_mock = mock.Mock()
    monkeypatch.setattr("geoipdb_loader._download_file", download_file_mock)
    geoipdb_loader.download()
    download_file_mock.assert_any_call(
        maxmind_filename="GeoLite2-City.mmdb.gz", skip_md5=False, local_filename=None
    )
    download_file_mock.assert_any_call(
        maxmind_filename="GeoLite2-Country.mmdb.gz", skip_md5=False, local_filename=None
    )
    assert download_file_mock.call_count == 2



from django.core.management.base import BaseCommand

from ... import download


class Logger(object):
    def __init__(self, style, stdout):
        self.style = style
        self.stdout = stdout

    def warn(self, template, *args):
        self.stdout.write(self.style.WARNING(template % args))

    def info(self, template, *args):
        self.stdout.write(template % args)


class Command(BaseCommand):
    help = "Download or update maxmind geoip db"

    def add_arguments(self, parser):
        parser.add_argument(
            "--skip-city",
            action="store_true",
            dest="skip_city",
            help="Don't download city db",
        )
        parser.add_argument(
            "--skip-country",
            action="store_true",
            dest="skip_country",
            help="Don't download country db",
        )
        parser.add_argument(
            "--skip-md5",
            action="store_true",
            dest="skip_md5",
            help="Don't check md5 sum of the downloaded files",
        )

    def handle(self, *args, **options):
        download(
            skip_city=options["skip_city"],
            skip_country=options["skip_country"],
            skip_md5=options["skip_md5"],
            logger=Logger(self.style, self.stdout),
        )



from __future__ import absolute_import

import io
import os
import re
from glob import glob

from setuptools import find_packages, setup


def read(*names, **kwargs):
    return io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8"),
    ).read()


setup(
    name="django-geoipdb-loader",
    version="0.9.0",
    description="Helps download and keep updated maxmind's geoip db required for django GeoIP",
    long_description="%s"
    % (
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
            "", read("README.rst")
        ),
    ),
    author="Konstantin Alekseev",
    author_email="kv.alekseev@gmail.com",
    url="https://github.com/kalekseev/django-geoipdb-loader",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[
        os.path.splitext(os.path.basename(path))[0] for path in glob("src/*.py")
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    keywords=["django", "geoip"],
)



import errno
import gzip
import hashlib
import logging
import os
import shutil
import tempfile

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.six.moves import urllib

MAXMIND_URL = "http://geolite.maxmind.com/download/geoip/database/"


def _match_md5(fp, md5_url):
    md5 = urllib.request.urlopen(md5_url).read()
    m = hashlib.md5()
    for line in fp:
        m.update(line)
    fp.seek(0)
    return m.hexdigest() == md5.decode()


def _atomic_write(fp, dst):
    tmpfile = tempfile.NamedTemporaryFile(delete=False, dir=os.path.dirname(dst))
    shutil.copyfileobj(fp, tmpfile.file)
    tmpfile.file.flush()
    os.fsync(tmpfile.file.fileno())
    tmpfile.file.close()
    try:
        os.rename(tmpfile.name, dst)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise
        os.unlink(dst)
        os.rename(tmpfile.name, dst)


def _download_file(maxmind_filename, skip_md5=False, local_filename=None):
    _, filename = os.path.split(maxmind_filename)
    if not os.path.exists(settings.GEOIP_PATH):
        raise ImproperlyConfigured("GEOIP_PATH directory doesn't exist on filesystem.")
    downloaded_file = os.path.join(settings.GEOIP_PATH, filename)
    if local_filename:
        db_file = os.path.join(settings.GEOIP_PATH, local_filename)
    else:
        db_file = os.path.splitext(downloaded_file)[0]
    urllib.request.urlretrieve(
        urllib.parse.urljoin(MAXMIND_URL, maxmind_filename), downloaded_file
    )
    with gzip.open(downloaded_file, "rb") as outfile:
        if not skip_md5:
            md5_url = urllib.parse.urljoin(
                MAXMIND_URL, filename.split(".", 1)[0] + ".md5"
            )
            if not _match_md5(outfile, md5_url):
                try:
                    os.remove(downloaded_file)
                finally:
                    raise ValueError(
                        "md5 of %s doesn't match the signature." % downloaded_file
                    )
        _atomic_write(outfile, db_file)
    os.remove(downloaded_file)


def _get_logger():
    return logging.getLogger(__name__)


def download(skip_city=False, skip_country=False, skip_md5=False, logger=None):
    logger = logger or _get_logger()
    files = []
    if not hasattr(settings, "GEOIP_PATH"):
        raise ImproperlyConfigured("GEOIP_PATH must be configured in settings.")
    if not skip_city:
        city_file = {
            "maxmind_filename": "GeoLite2-City.mmdb.gz",
            "skip_md5": skip_md5,
            "local_filename": getattr(settings, "GEOIP_CITY", None),
        }
        files.append(city_file)
    if not skip_country:
        country_file = {
            "maxmind_filename": "GeoLite2-Country.mmdb.gz",
            "skip_md5": skip_md5,
            "local_filename": getattr(settings, "GEOIP_COUNTRY", None),
        }
        files.append(country_file)
    if not files:
        logger.warn("Nothing to download.")
        return
    for entry in files:
        logger.info("Downloading %s ..." % entry["maxmind_filename"])
        _download_file(**entry)
    logger.info("Geoip files are updated.")
